﻿define([
// dojo
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dojo/dom-class",
    "dojo/string",

// dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",

// epi
    "epi/shell/widget/dialog/Alert",
    "epi/shell/widget/_ModelBindingMixin",

    "./ProjectSelector",
    "./viewmodels/ProjectModeToolbarViewModel",

// template
    "dojo/text!./templates/ProjectModeToolbar.html",
    "dojo/text!./templates/CurrentProjectDoesNotExistsWarning.html",

// Resources
    "epi/i18n!epi/cms/nls/episerver.cms.components.project.toolbar",
    "epi/i18n!epi/nls/episerver.shared"
], function (
// dojo
    declare,
    lang,

    domClass,
    string,

    // dijit
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _LayoutWidget,

// epi
    Alert,
    _ModelBindingMixin,

    ProjectSelector,
    ProjectModeToolbarViewModel,

// template
    template,
    projectDoesNotExistsWarningTemplate,

// Resources
    localizations,
    sharedResources
) {

    return declare([_LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin, _ModelBindingMixin], {
        // summary:
        //      Project mode toolbar for selecting the project to work with
        // tags:
        //      internal

        templateString: template,

        localizations: localizations,

        _projectDoesNotExistsWarning: "",

        postMixInProperties: function () {
            this.inherited(arguments);

            if (!this.model) {
                this.own(
                    this.model = new ProjectModeToolbarViewModel()
                );
            }

            this._projectDoesNotExistsWarning = string.substitute(projectDoesNotExistsWarningTemplate, {
                warning: localizations.projectdoesnotexists.alert.warning, 
                description: localizations.projectdoesnotexists.alert.description
            });
        },

        buildRendering: function() {
            this.inherited(arguments);

            this.own(this.projectSelector = new ProjectSelector({
                iconClass: "dijitNoIcon",
                "class": "epi-chromeless epi-chromeless--inverted epi-chromeless--with-arrow",
                enableDefaultValue: true,
                defaultValueText: localizations.defaultoption
            }, this.projectSelectorNode));

            var domNode = this.domNode,
                dialogContent = this._projectDoesNotExistsWarning;
            
            this.own(this.model.on("currentProjectDoesNotExists", function() {
                domClass.add(domNode, "epi-project-mode-toolbar--project-deleted");
                var confirm = new Alert({
                    destroyOnHide: true,
                    description: dialogContent
                });

                confirm.show();
            }));
        },

        startup: function() {
            this.inherited(arguments);

            this.projectSelector.set("store", this.model.projectStore);

            var model = this.model;
            this.model.initialize().then(lang.hitch(this, function() {
                var domNode = this.domNode;

                this.own(this.projectSelector.watch("value", function(property, oldValue, newValue) {
                    domClass.toggle(domNode, "epi-project-mode-toolbar--project-selected", newValue);
                    domClass.remove(domNode, "epi-project-mode-toolbar--project-deleted");

                    model.set("currentProject", newValue);
                }));

                this.projectSelector.set("value", model.get("currentProject"));
            }));
        }
    });
});