define([
        // dojo
        "dojo/_base/declare",
        "dojo/_base/lang",
        "dojo/Deferred",
        "dojo/Evented",
        "dojo/Stateful",
        "dojo/topic",
        "dojo/when",
        // dijit
        "dijit/Destroyable",

        //epi
        "epi/epi",
        "epi/dependency",
        "epi-cms/_ContentContextMixin"
    ],

    function(
        // dojo
        declare,
        lang,
        Deferred,
        Evented,
        Stateful,
        topic,
        when,
        // dijit
        Destroyable,

        // epi
        epi,
        dependency,
        _ContentContextMixin
    ) {
        return declare([Stateful, Destroyable, Evented, _ContentContextMixin], {
            // summary:
            //      The view model for the epi-cms/project/ProjectModeToolbar
            // tags:
            //      internal

            // projectStore: [readonly] Store
            //      A REST store for interacting with projects.
            projectStore: null,

            // currentProject: [public] Object
            //      The project that is currently selected.
            currentProject: null,

            postscript: function() {
                this.inherited(arguments);

                // Resolve the project service and store from the dependency if they
                // haven't been injected. This allows us to mock them when testing.
                this.projectService = this.projectService || dependency.resolve("epi.cms.ProjectService");
                this.projectStore = this.projectStore || dependency.resolve("epi.storeregistry").get("epi.cms.project");
            },

            initialize: function() {
                // summary:
                //
                // returns: Promise
                //
                // tags:
                //      public

                // Set the current project based on what is stored in the profile
                return when(this.projectService.getCurrentProject()).then(lang.hitch(this, function(project) {
                    this._changeAttrValue("currentProject", project, false); // use the internal setter to not trigger a profile save
                }));
            },
            contentContextChanged: function(ctx, callerData) {
                // summary:
                //      Callback method for when the current content context has changed
                // tags:
                //      protected

                if(this.get("currentProject")) {
                    var self = this;
                    this.projectService.exists(this.currentProject.id).then(function(exists) {
                        if(!exists) {
                            self.emit("currentProjectDoesNotExists");
                        }
                    });
                }
            },

            _currentProjectSetter: function(currentProject, persist) {
                // Early exit if the project has not changed
                if (epi.areEqual(this.currentProject, currentProject)) {
                    return;
                }

                this.currentProject = currentProject;

                var promise = new Deferred().resolve();

                // Store the selected project.
                if (persist || persist === undefined) {
                    promise = this.projectService.setCurrentProject(currentProject);
                }

                // Refresh the current context when switching projects to ensure
                // that the preview is correct.
                promise
                    .then(lang.hitch(this, "getCurrentContext"))
                    .then(function(context) {
                        topic.publish("/epi/shell/context/request", { uri: context.versionAgnosticUri }, { sender: this });
                    });
            }

        });
    });
